package maze.model;

// Provide MazeTest only

public class Maze {
	private Cell[][] cells;
	
	public Maze(int width, int height) {
		cells = new Cell[width][height];
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				cells[x][y] = Cell.CLOSED;
			}
		}
	}
	
	public int getWidth() {return cells.length;}
	public int getHeight() {return cells[0].length;}
	
	public void clearVisits() {
		for (int x = 0; x < getWidth(); x++) {
			for (int y = 0; y < getHeight(); y++) {
				if (cells[x][y] == Cell.VISITED) {
					cells[x][y] = Cell.OPEN;
				}
			}
		}
	}
	
	public int getCountOf(Cell cellType) {
		int count = 0;
		for (int x = 0; x < getWidth(); x++) {
			for (int y = 0; y < getHeight(); y++) {
				if (cells[x][y] == cellType) {count += 1;}
			}
		}
		return count;
	}
	
	public boolean inMaze(Position p) {
		return p.getX() >= 0 && p.getX() < getWidth() && p.getY() >= 0 && p.getY() < getHeight();
	}
	
	public Cell getStateFor(Position p) {
		return inMaze(p) ? cells[p.getX()][p.getY()] : Cell.CLOSED;
	}
	
	public void setStateFor(Position p, Cell state) {
		cells[p.getX()][p.getY()] = state;
	}
	
	public boolean inRoom(Position p) {
		if (!getStateFor(p).canEnter()) {
			return false;
		}
		for (Direction d: Direction.values()) {
			if (openL(p, d)) {
				return true;
			}
		}
		return false;
	}
	
	public int numOpenNeighbors(Position p) {
		int count = 0;
		for (Direction d: Direction.values()) {
			if (getStateFor(d.getNeighbor(p)).canEnter()) {
				count += 1;
			}
		}
		return count;
	}
	
	private boolean openL(Position p, Direction d) {
		Position n1 = d.getNeighbor(p);
		Direction d2 = d.getClockwise();
		Position n2 = d2.getNeighbor(p);
		Position n3 = d2.getNeighbor(n1);
		return getStateFor(n1).canEnter() && getStateFor(n2).canEnter() && getStateFor(n3).canEnter();
	}
}
