package maze.model;

import java.util.ArrayList;

import javafx.scene.paint.Color;
import maze.searchers.Searcher;

// Provide PuzzleTest only

public class Puzzle {
	private Maze dungeon;
	private Explorer hero;
	private Position goal;
	
	public Puzzle(int width, int height) {
		dungeon = new Maze(width, height);
	}
	
	public int getWidth() {return dungeon.getWidth();}
	public int getHeight() {return dungeon.getHeight();} 
	
	public void tunnelRandomly() {
		ArrayList<Position> stack = new ArrayList<>();
		stack.add(new Position(0, 0));
		while (!stack.isEmpty()) {
			Position where = stack.remove(stack.size() - 1);
			if (dungeon.inMaze(where) && !canEnter(where) && dungeon.numOpenNeighbors(where) <= 1) {
				clear(where);
				for (Direction d: Direction.randomDirections()) {
					stack.add(d.getNeighbor(where));
				}
			}
		}
	}

	public boolean clear(Position p) {
		if (dungeon.inMaze(p)) {
			dungeon.setStateFor(p, Cell.OPEN);
			if (dungeon.inRoom(p)) {
				dungeon.setStateFor(p, Cell.CLOSED);
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}
	
	public void shotgun(double fraction) {
		int numCells = (int)(fraction * dungeon.getWidth() * dungeon.getHeight());
		for (int i = 0; i < numCells; i++) {
			int x = (int)(Math.random() * dungeon.getWidth());
			int y = (int)(Math.random() * dungeon.getHeight());
			dungeon.setStateFor(new Position(x, y), Cell.OPEN);
		}
	}
	
	public void fill(Position p) {
		dungeon.setStateFor(p, Cell.CLOSED);
	}
	
	public boolean canEnter(Position p) {
		return dungeon.getStateFor(p).canEnter();
	}
	
	public Color getColorFor(Position p) {
		return dungeon.getStateFor(p).getColor();
	}
	
	public boolean hasExplorer() {return hero != null;}
	
	public void placeExplorer(Position p) {
		checkEnter(p);
		hero = new Explorer(p);
	}
	
	public void moveExplorer(Move m) {
		hero.move(m, dungeon);
	}
	
	public Position getExplorerPosition() {
		return hero.getLocation();
	}
	
	public Direction getExplorerHeading() {
		return hero.getHeading();
	}
	
	public void placeGoal(Position p) {
		checkEnter(p);
		goal = p;
	}
	
	public void clearVisits() {
		dungeon.clearVisits();
	}
	
	public int getCountOf(Cell cellType) {
		return dungeon.getCountOf(cellType);
	}
	
	public Trail solve(Searcher<Trail> solver) {
		if (!hasExplorer() || !hasGoal()) {return null;}
		
		solver.add(new Trail(hero.getLocation(), null));
		while (!solver.isEmpty() && !solver.peek().getEnd().equals(goal)) {
			Trail at = solver.remove();
			if (dungeon.getStateFor(at.getEnd()) == Cell.OPEN) {
				dungeon.setStateFor(at.getEnd(), Cell.VISITED);
				for (Direction d: Direction.values()) {
					solver.add(new Trail(d.getNeighbor(at.getEnd()), at));
				}
			}
		}
		return solver.isEmpty() ? null : solver.peek();
	}
	
	public boolean hasGoal() {return goal != null;}
	
	public Position getGoal() {return goal;}
	
	private void checkEnter(Position p) {
		if (!canEnter(p)) {
			throw new IllegalArgumentException("Can't enter " + p);
		}
	}
}
